use reqwest::{blocking, Method};
use std::{result::Result, error::Error, process::exit, io::{Write, stderr}, fs, path::Path};

fn write_metrics(add: u32) -> Result<(), Box<dyn Error>> {
    let user = users::get_user_by_uid(users::get_current_uid());
    if let Some(user) = user.clone() {
        let path = format!("/home/{}/.local/share/bHeartbeat", user.name().to_str().unwrap());
        let file_path = path.clone() + "/data_spent.txt";
        let _ = fs::create_dir_all(path);
        if ! Path::new(&file_path).is_file() {
            fs::write(&file_path, add.to_string())?;
        } else {
            let f = fs::File::open(&file_path)?;
            let meta = f.metadata()?;
            if meta.permissions().readonly() {
                return Err("Can't write to the metrics file".into());
            }

            let mut data = fs::read_to_string(&file_path)?.parse::<u32>().unwrap_or(0);
            data += add;
            fs::write(&file_path, data.to_string())?;
        }
    } else {
        return Err("No user data available. Metrics will not be saved.".into())
    }
    Ok(())
}

fn main() -> Result<(), Box<dyn Error>> {
    let key = include_bytes!("key.bin");
    let cli = blocking::Client::new();
    let res = cli.request(Method::POST, "https://hb.blek.codes/beat")
        .body(key.to_vec())
        .send()?;
    
    if res.status() != 200 {
        eprintln!("HTTP heartbeat returned {}", res.status());
        exit(1);
    }

    let data = write_metrics((1024 * 10) + key.len() as u32);
    if let Err(data) = data {
        eprintln!("Can't write metrics:\n{}", data.to_string());
    }

    let status = res.bytes()?.to_vec();
    let status = status.as_slice();
    let ok_status = &status[0..2];

    if ok_status != "OK".as_bytes() {
        eprintln!("ERR: Server returned this:");
        eprintln!("-----BEGIN BLOCK-----");
        stderr().write(status)?;
        eprintln!("\n-----END BLOCK-----");
    }

    Ok(())
}
